package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.FixationArtifactAccess;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.CalculationResult;

import de.intevation.flys.utils.KMIndex;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;


/**
 * Facet to show average W values for Q sectors.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixAvSectorFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixAvSectorFacet.class);

    /** Trivial Constructor. */
    public FixAvSectorFacet() {
    }


    public FixAvSectorFacet(int ndx, String name, String description) {
        super(
            ndx,
            name,
            description,
            ComputeType.ADVANCE,
            null,
            null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext.
     *
     * @return the data as QWD array (QWD[]).
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixAvSectorFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixationArtifactAccess access = new FixationArtifactAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();

            double km = access.getCurrentKm();

            logger.debug("Current KM = " + km);

            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();
            KMIndex.Entry<AnalysisPeriod []> kmPeriodsEntry =
                kmPeriods.search(km);

            if (kmPeriodsEntry == null) {
                logger.warn("No analysis periods found for km '" + km + "'");
                return null;
            }

            AnalysisPeriod[] periods = kmPeriodsEntry.getValue();

            if (periods == null) {
                logger.warn("No analysis periods specified!");
                return null;
            }

            QWD[] qwdData = periods[index].getQSectorAverages();

            if (logger.isDebugEnabled()) {
                int resSize = qwdData != null ? qwdData.length : -1;
                logger.debug("Found " + resSize + " result elements.");
            }

            return qwdData;
        }
        else {
            logger.warn("Artifact is no instance of FLYSArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixAvSectorFacet deepCopy() {
        FixAvSectorFacet copy = new FixAvSectorFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
